@@ -77,10 +77,13 @@ class AndroidBoardConfig(object):
         """
         boot_env = {}
         boot_env["bootargs"] = cls._get_bootargs(consoles)
+        initrd = False
+        if cls.initrd_addr:
+            initrd = True
         # On Android, the DTB file is always built as part of the kernel it
         # comes from - and lives in the same directory in the boot tarball, so
         # here we don't need to pass the whole path to it.
-        boot_env["bootcmd"] = cls._get_bootcmd(cls.dtb_name)
+        boot_env["bootcmd"] = cls._get_bootcmd(initrd, cls.dtb_name)
         return boot_env
 
     @classmethod
@@ -189,12 +192,14 @@ class AndroidBeagleConfig(AndroidOmapConfig, BeagleConfig):
 
 
 class AndroidPandaConfig(AndroidOmapConfig, PandaConfig):
+    bootloader_flavor = 'omap4_panda'
+    dtb_addr = '0x815f0000'
+    dtb_name = 'board.dtb'
     _extra_serial_opts = 'console=ttyO2,115200n8'
     extra_boot_args_options = (
         'earlyprintk fixrtc nocompcache vram=48M '
         'omapfb.vram=0:24M,1:24M mem=456M@0x80000000 mem=512M@0xA0000000')
     android_specific_args = 'init=/init androidboot.console=ttyO2'
-    dtb_name = None
 
 
 class AndroidSnowballSdConfig(AndroidBoardConfig, SnowballSdConfig):
@@ -202,7 +207,7 @@ class AndroidSnowballSdConfig(AndroidBoardConfig, SnowballSdConfig):
     initrd_addr = '0x05000000'
     extra_boot_args_options = (
         'earlyprintk mem=128M@0 mali.mali_mem=64M@128M hwmem=168M@192M '
-        'mem=22M@360M mem_issw=1M@383M mem=640M@384M vmalloc=300M')
+        'mem=22M@360M mem_issw=1M@383M mem=640M@384M vmalloc=500M')
     _extra_serial_opts = 'console=ttyAMA2,115200n8'
     android_specific_args = 'init=/init androidboot.console=ttyAMA2'
     dtb_name = None
@@ -213,7 +218,7 @@ class AndroidSnowballEmmcConfig(AndroidBoardConfig, SnowballEmmcConfig):
     initrd_addr = '0x05000000'
     extra_boot_args_options = (
         'earlyprintk mem=128M@0 mali.mali_mem=64M@128M hwmem=168M@192M '
-        'mem=22M@360M mem_issw=1M@383M mem=640M@384M vmalloc=300M')
+        'mem=22M@360M mem_issw=1M@383M mem=640M@384M vmalloc=500M')
     _extra_serial_opts = 'console=ttyAMA2,115200n8'
     android_specific_args = 'init=/init androidboot.console=ttyAMA2'
     mmc_option = '0:2'
@@ -296,7 +301,7 @@ class AndroidMx53LoCoConfig(AndroidBoardConfig, Mx53LoCoConfig):
 
 
 class AndroidMx6QSabreliteConfig(AndroidMx53LoCoConfig):
-    uboot_flavor = 'mx6qsabrelite'
+    bootloader_flavor = 'mx6qsabrelite'
     kernel_addr = '0x10000000'
     initrd_addr = '0x12000000'
     load_addr = '0x10008000'
