@@ -36,6 +36,7 @@ import ConfigParser
 import shutil
 import string
 import logging
+from linaro_image_tools.hwpack.config import Config
 
 from parted import Device
 
@@ -43,7 +44,7 @@ from linaro_image_tools import cmd_runner
 
 from linaro_image_tools.media_create.partitions import (
     partition_mounted, SECTOR_SIZE, register_loopback)
-
+from StringIO import StringIO
 
 KERNEL_GLOB = 'vmlinuz-*-%(kernel_flavor)s'
 INITRD_GLOB = 'initrd.img-*-%(kernel_flavor)s'
@@ -114,6 +115,7 @@ class classproperty(object):
 class HardwarepackHandler(object):
     FORMAT_1 = '1.0'
     FORMAT_2 = '2.0'
+    FORMAT_3 = '3.0'
     FORMAT_MIXED = '1.0and2.0'
     metadata_filename = 'metadata'
     format_filename = 'FORMAT'
@@ -121,9 +123,11 @@ class HardwarepackHandler(object):
     hwpack_tarfiles = []
     tempdir = None
 
-    def __init__(self, hwpacks):
+    def __init__(self, hwpacks, bootloader=None, board=None):
         self.hwpacks = hwpacks
         self.hwpack_tarfiles = []
+        self.bootloader = bootloader
+        self.board = board
 
     class FakeSecHead(object):
         """ Add a fake section header to the metadata file.
@@ -158,18 +162,19 @@ class HardwarepackHandler(object):
         if self.tempdir is not None and os.path.exists(self.tempdir):
             shutil.rmtree(self.tempdir)
 
-    def get_field(self, section, field):
+    def get_field(self, field):
         data = None
         hwpack_with_data = None
         for hwpack_tarfile in self.hwpack_tarfiles:
             metadata = hwpack_tarfile.extractfile(self.metadata_filename)
-            # Use RawConfigParser which does not support the magical
-            # interpolation behavior of ConfigParser so we don't mess up
-            # metadata accidentally.
-            parser = ConfigParser.RawConfigParser()
-            parser.readfp(self.FakeSecHead(metadata))
+            lines = metadata.readlines()
+            if re.search("=", lines[0]) and not re.search(":", lines[0]):
+                # Probably V2 hardware pack without [hwpack] on the first line
+                lines = ["[hwpack]\n"] + lines
+            parser = Config(StringIO("".join(lines)), self.bootloader,
+                            self.board)
             try:
-                new_data = parser.get(section, field)
+                new_data = parser.get_option(field)
                 if new_data is not None:
                     assert data is None, "The metadata field '%s' is set to " \
                         "'%s' and new value '%s' is found" % (field, data,
@@ -178,11 +183,12 @@ class HardwarepackHandler(object):
                     hwpack_with_data = hwpack_tarfile
             except ConfigParser.NoOptionError:
                 continue
+
         return data, hwpack_with_data
 
     def get_format(self):
         format = None
-        supported_formats = [self.FORMAT_1, self.FORMAT_2]
+        supported_formats = [self.FORMAT_1, self.FORMAT_2, self.FORMAT_3]
         for hwpack_tarfile in self.hwpack_tarfiles:
             format_file = hwpack_tarfile.extractfile(self.format_filename)
             format_string = format_file.read().strip()
@@ -196,8 +202,7 @@ class HardwarepackHandler(object):
         return format
 
     def get_file(self, file_alias):
-        file_name, hwpack_tarfile = self.get_field(self.main_section,
-                                                   file_alias)
+        file_name, hwpack_tarfile = self.get_field(file_alias)
         if file_name is not None:
             hwpack_tarfile.extract(file_name, self.tempdir)
             file_name = os.path.join(self.tempdir, file_name)
@@ -209,10 +214,10 @@ class BoardConfig(object):
     """The configuration used when building an image for a board."""
     hwpack_format = None
     # These attributes may not need to be redefined on some subclasses.
-    uboot_flavor = None
+    bootloader_flavor = None
     # whether to copy u-boot to the boot partition
-    uboot_in_boot_part = False
-    uboot_dd = False
+    bootloader_file_in_boot_part = False
+    bootloader_dd = False
     spl_in_boot_part = False
     spl_dd = False
     env_dd = False
@@ -289,8 +294,7 @@ class BoardConfig(object):
     def get_metadata_field(cls, field_name):
         """ Return the metadata value for field_name if it can be found.
         """
-        data, _ = cls.hardwarepack_handler.get_field(
-            cls.hardwarepack_handler.main_section, field_name)
+        data, _ = cls.hardwarepack_handler.get_field(field_name)
         return data
 
     @classmethod
@@ -298,14 +302,15 @@ class BoardConfig(object):
         cls.board = board
 
     @classmethod
-    def set_metadata(cls, hwpacks):
-        cls.hardwarepack_handler = HardwarepackHandler(hwpacks)
+    def set_metadata(cls, hwpacks, bootloader=None, board=None):
+        cls.hardwarepack_handler = HardwarepackHandler(hwpacks, bootloader,
+                                                       board)
         with cls.hardwarepack_handler:
             cls.hwpack_format = cls.hardwarepack_handler.get_format()
             if (cls.hwpack_format == cls.hardwarepack_handler.FORMAT_1):
                 return
 
-            if (cls.hwpack_format == cls.hardwarepack_handler.FORMAT_2):
+            if (cls.hwpack_format != cls.hardwarepack_handler.FORMAT_1):
                 # Clear V1 defaults.
                 cls.kernel_addr = None
                 cls.initrd_addr = None
@@ -334,19 +339,19 @@ class BoardConfig(object):
             cls.serial_tty = cls.get_metadata_field('serial_tty')
             wired_interfaces = cls.get_metadata_field('wired_interfaces')
             if wired_interfaces is not None:
-                cls.wired_interfaces = wired_interfaces.split(' ')
+                cls.wired_interfaces = wired_interfaces
             wireless_interfaces = cls.get_metadata_field(
                 'wireless_interfaces')
             if wireless_interfaces is not None:
-                cls.wireless_interfaces = wireless_interfaces.split(' ')
-            cls.vmlinuz = cls.get_metadata_field('kernel_file')
-            cls.initrd = cls.get_metadata_field('initrd_file')
+                cls.wireless_interfaces = wireless_interfaces
+            cls.vmlinuz = cls.get_metadata_field('vmlinuz')
+            cls.initrd = cls.get_metadata_field('initrd')
             cls.dtb_file = cls.get_metadata_field('dtb_file')
             cls.extra_boot_args_options = cls.get_metadata_field(
                 'extra_boot_options')
             cls.boot_script = cls.get_metadata_field('boot_script')
             cls.extra_serial_opts = cls.get_metadata_field(
-                'extra_serial_options')
+                'extra_serial_opts')
             cls.snowball_startup_files_config = cls.get_metadata_field(
                 'snowball_startup_files_config')
 
@@ -379,13 +384,14 @@ class BoardConfig(object):
                     align_up(int(loader_min_size) * 1024 ** 2,
                              SECTOR_SIZE) / SECTOR_SIZE)
 
-            uboot_in_boot_part = cls.get_metadata_field('u_boot_in_boot_part')
-            if uboot_in_boot_part is None:
-                cls.uboot_in_boot_part = False
-            elif string.lower(uboot_in_boot_part) == 'yes':
-                cls.uboot_in_boot_part = True
-            elif string.lower(uboot_in_boot_part) == 'no':
-                cls.uboot_in_boot_part = False
+            bootloader_file_in_boot_part = cls.get_metadata_field(
+                'bootloader_file_in_boot_part')
+            if bootloader_file_in_boot_part is None:
+                cls.bootloader_file_in_boot_part = False
+            elif string.lower(bootloader_file_in_boot_part) == 'yes':
+                cls.bootloader_file_in_boot_part = True
+            elif string.lower(bootloader_file_in_boot_part) == 'no':
+                cls.bootloader_file_in_boot_part = False
             spl_in_boot_part = cls.get_metadata_field('spl_in_boot_part')
             if spl_in_boot_part is None:
                 cls.spl_in_boot_part = False
@@ -401,12 +407,13 @@ class BoardConfig(object):
             elif string.lower(env_dd) == 'no':
                 cls.env_dd = False
 
-            uboot_dd = cls.get_metadata_field('u_boot_dd')
-            # Either uboot_dd is not specified, or it contains the dd offset.
-            if uboot_dd is None:
-                cls.uboot_dd = False
+            bootloader_dd = cls.get_metadata_field('bootloader_dd')
+            # Either bootloader_dd is not specified, or it contains the dd
+            # offset.
+            if bootloader_dd is None:
+                cls.bootloader_dd = False
             else:
-                cls.uboot_dd = int(uboot_dd)
+                cls.bootloader_dd = int(bootloader_dd)
             spl_dd = cls.get_metadata_field('spl_dd')
             # Either spl_dd is not specified, or it contains the dd offset.
             if spl_dd is None:
@@ -569,7 +576,7 @@ class BoardConfig(object):
             return cls.get_v1_sfdisk_cmd(should_align_boot_part)
 
     @classmethod
-    def _get_bootcmd(cls, d_img_data):
+    def _get_bootcmd(cls, i_img_data, d_img_data):
         """Get the bootcmd for this board.
 
         In general subclasses should not have to override this.
@@ -580,20 +587,22 @@ class BoardConfig(object):
             initrd_addr=cls.initrd_addr, dtb_addr=cls.dtb_addr)
         boot_script = (
             ("%(fatload_command)s mmc %(mmc_option)s %(kernel_addr)s " +
-             "%(uimage_path)suImage; ") +
+             "%(uimage_path)suImage; ")) % replacements
+        if i_img_data is not None:
+            boot_script += (
             ("%(fatload_command)s mmc %(mmc_option)s %(initrd_addr)s " +
              "%(uimage_path)suInitrd; ")) % replacements
-        if d_img_data is not None:
-            assert cls.dtb_addr is not None, (
-                "Need a dtb_addr when passing d_img_data")
-            boot_script += (
-                ("%(fatload_command)s mmc %(mmc_option)s %(dtb_addr)s " +
-                 "board.dtb; ") +
-                "bootm %(kernel_addr)s %(initrd_addr)s %(dtb_addr)s"
-                ) % replacements
-        else:
-            boot_script += (
-                "bootm %(kernel_addr)s %(initrd_addr)s" % replacements)
+            if d_img_data is not None:
+                assert cls.dtb_addr is not None, (
+                    "Need a dtb_addr when passing d_img_data")
+                boot_script += (
+                    ("%(fatload_command)s mmc %(mmc_option)s %(dtb_addr)s " +
+                     "board.dtb; ")) % replacements
+        boot_script += (("bootm %(kernel_addr)s")) % replacements
+        if i_img_data is not None:
+            boot_script += ((" %(initrd_addr)s")) % replacements
+            if d_img_data is not None:
+                boot_script += ((" %(dtb_addr)s")) % replacements
         return boot_script
 
     @classmethod
@@ -611,7 +620,7 @@ class BoardConfig(object):
                 cls.add_boot_args(boot_args_file.read().strip())
 
     @classmethod
-    def _get_bootargs(cls, is_live, is_lowmem, consoles, rootfs_uuid):
+    def _get_bootargs(cls, is_live, is_lowmem, consoles, rootfs_id):
         """Get the bootargs for this board.
 
         In general subclasses should not have to override this.
@@ -624,7 +633,7 @@ class BoardConfig(object):
             serial_opts += ' console=%s' % console
 
         lowmem_opt = ''
-        boot_snippet = 'root=UUID=%s' % rootfs_uuid
+        boot_snippet = 'root=%s' % rootfs_id
         if is_live:
             serial_opts += ' %s' % cls.live_serial_opts
             boot_snippet = 'boot=casper'
@@ -641,30 +650,30 @@ class BoardConfig(object):
              % replacements)
 
     @classmethod
-    def _get_boot_env(cls, is_live, is_lowmem, consoles, rootfs_uuid,
-                      d_img_data):
+    def _get_boot_env(cls, is_live, is_lowmem, consoles, rootfs_id,
+                      i_img_data, d_img_data):
         """Get the boot environment for this board.
 
         In general subclasses should not have to override this.
         """
         boot_env = {}
         boot_env["bootargs"] = cls._get_bootargs(
-            is_live, is_lowmem, consoles, rootfs_uuid)
-        boot_env["bootcmd"] = cls._get_bootcmd(d_img_data)
+            is_live, is_lowmem, consoles, rootfs_id)
+        boot_env["bootcmd"] = cls._get_bootcmd(i_img_data, d_img_data)
         return boot_env
 
     @classmethod
-    def make_boot_files(cls, uboot_parts_dir, is_live, is_lowmem, consoles,
-                        chroot_dir, rootfs_uuid, boot_dir,
+    def make_boot_files(cls, bootloader_parts_dir, is_live, is_lowmem,
+                        consoles, chroot_dir, rootfs_id, boot_dir,
                         boot_device_or_file):
         if cls.hwpack_format == HardwarepackHandler.FORMAT_1:
-            parts_dir = uboot_parts_dir
+            parts_dir = bootloader_parts_dir
         else:
             parts_dir = chroot_dir
         (k_img_data, i_img_data, d_img_data) = cls._get_kflavor_files(
             parts_dir)
-        boot_env = cls._get_boot_env(is_live, is_lowmem, consoles, rootfs_uuid,
-                                     d_img_data)
+        boot_env = cls._get_boot_env(is_live, is_lowmem, consoles, rootfs_id,
+                                     i_img_data, d_img_data)
 
         if cls.hwpack_format == HardwarepackHandler.FORMAT_1:
             cls._make_boot_files(
@@ -686,12 +695,12 @@ class BoardConfig(object):
         _dd(from_file, to_file, seek=seek)
 
     @classmethod
-    def install_samsung_boot_loader(cls, samsung_spl_file, uboot_file,
+    def install_samsung_boot_loader(cls, samsung_spl_file, bootloader_file,
                                     boot_device_or_file):
                 cls._dd_file(samsung_spl_file, boot_device_or_file,
                              cls.SAMSUNG_V310_BL1_START,
                              cls.SAMSUNG_V310_BL1_LEN * SECTOR_SIZE)
-                cls._dd_file(uboot_file, boot_device_or_file,
+                cls._dd_file(bootloader_file, boot_device_or_file,
                              cls.SAMSUNG_V310_BL2_START,
                              cls.SAMSUNG_V310_BL2_LEN * SECTOR_SIZE)
 
@@ -700,7 +709,7 @@ class BoardConfig(object):
                          boot_device_or_file, k_img_data, i_img_data,
                          d_img_data):
         with cls.hardwarepack_handler:
-            spl_file = cls.get_file('spl')
+            spl_file = cls.get_file('spl_file')
             if cls.spl_in_boot_part:
                 assert spl_file is not None, (
                     "SPL binary could not be found")
@@ -715,12 +724,15 @@ class BoardConfig(object):
             if cls.spl_dd:
                 cls._dd_file(spl_file, boot_device_or_file, cls.spl_dd)
 
-            uboot_file = cls.get_file('u_boot')
-            if cls.uboot_dd:
-                cls._dd_file(uboot_file, boot_device_or_file, cls.uboot_dd)
+            bootloader_file = cls.get_file('bootloader_file')
+            if cls.bootloader_dd:
+                cls._dd_file(bootloader_file, boot_device_or_file,
+                             cls.bootloader_dd)
 
         make_uImage(cls.load_addr, k_img_data, boot_dir)
-        make_uInitrd(i_img_data, boot_dir)
+
+        if i_img_data is not None:
+            make_uInitrd(i_img_data, boot_dir)
 
         if d_img_data is not None:
             make_dtb(d_img_data, boot_dir)
@@ -758,40 +770,40 @@ class BoardConfig(object):
         raise NotImplementedError()
 
     @classmethod
-    def populate_boot(cls, chroot_dir, rootfs_uuid, boot_partition, boot_disk,
+    def populate_boot(cls, chroot_dir, rootfs_id, boot_partition, boot_disk,
                       boot_device_or_file, is_live, is_lowmem, consoles):
         parts_dir = 'boot'
         if is_live:
             parts_dir = 'casper'
-        uboot_parts_dir = os.path.join(chroot_dir, parts_dir)
-
+        bootloader_parts_dir = os.path.join(chroot_dir, parts_dir)
         cmd_runner.run(['mkdir', '-p', boot_disk]).wait()
         with partition_mounted(boot_partition, boot_disk):
-            if cls.uboot_in_boot_part:
+            if cls.bootloader_file_in_boot_part:
                 with cls.hardwarepack_handler:
                     # <legacy v1 support>
-                    if cls.uboot_flavor is not None:
+                    if cls.bootloader_flavor is not None:
                         default = os.path.join(
                             chroot_dir, 'usr', 'lib', 'u-boot',
-                            cls.uboot_flavor, 'u-boot.img')
+                            cls.bootloader_flavor, 'u-boot.img')
                         if not os.path.exists(default):
                             default = os.path.join(
                                 chroot_dir, 'usr', 'lib', 'u-boot',
-                                cls.uboot_flavor, 'u-boot.bin')
+                                cls.bootloader_flavor, 'u-boot.bin')
                     else:
                         default = None
                     # </legacy v1 support>
-                    uboot_bin = cls.get_file('u_boot', default=default)
-                    assert uboot_bin is not None, (
-                        "uboot binary could not be found")
+                    bootloader_bin = cls.get_file('bootloader_file',
+                                                   default=default)
+                    assert bootloader_bin is not None, (
+                        "bootloader binary could not be found")
 
                     proc = cmd_runner.run(
-                        ['cp', '-v', uboot_bin, boot_disk], as_root=True)
+                        ['cp', '-v', bootloader_bin, boot_disk], as_root=True)
                     proc.wait()
 
             cls.make_boot_files(
-                uboot_parts_dir, is_live, is_lowmem, consoles, chroot_dir,
-                rootfs_uuid, boot_disk, boot_device_or_file)
+                bootloader_parts_dir, is_live, is_lowmem, consoles, chroot_dir,
+                rootfs_id, boot_disk, boot_device_or_file)
 
     @classmethod
     def _get_kflavor_files(cls, path):
@@ -823,22 +835,27 @@ class BoardConfig(object):
 
     @classmethod
     def _get_kflavor_files_v2(cls, path):
-        kernel = _get_file_matching(os.path.join(path, cls.vmlinuz))
-        if kernel is not None:
+        kernel = initrd = dtb = None
+        logger = logging.getLogger("linaro_image_tools")
+
+        if cls.vmlinuz:
+            kernel = _get_file_matching(os.path.join(path, cls.vmlinuz))
+        if not cls.vmlinuz or not kernel:
+            raise ValueError("Unable to find a valid kernel image.")
+
+        if cls.initrd:
             initrd = _get_file_matching(os.path.join(path, cls.initrd))
-            if initrd is not None:
-                dtb = None
-                if cls.dtb_file is not None:
-                    dtb = _get_file_matching(os.path.join(path, cls.dtb_file))
-                logger = logging.getLogger("linaro_image_tools")
-                logger.info("Will use kernel=%s, initrd=%s, dtb=%s." % \
-                                 (kernel, initrd, dtb))
-                return (kernel, initrd, dtb)
-            raise ValueError(
-                "Found kernel matching %s but no initrd matching %s" % (
-                    cls.vmlinuz, cls.initrd))
-        raise ValueError(
-            "No kernel found matching %s." % (cls.vmlinuz))
+        if not cls.initrd or not initrd:
+            logger.warn("Could not find a valid initrd, skipping uInitd.")
+
+        if cls.dtb_file:
+            dtb = _get_file_matching(os.path.join(path, cls.dtb_file))
+        if not cls.dtb_file or not dtb:
+            logger.warn("Could not find a valid dtb file, skipping it.")
+
+        logger.info("Will use kernel=%s, initrd=%s, dtb=%s." % \
+                             (kernel, initrd, dtb))
+        return (kernel, initrd, dtb)
 
     @classmethod
     def populate_raw_partition(cls, media, boot_dir):
@@ -855,7 +872,7 @@ class BoardConfig(object):
 
 class OmapConfig(BoardConfig):
     kernel_flavors = ['linaro-omap4', 'linaro-lt-omap', 'linaro-omap', 'omap4']
-    uboot_in_boot_part = True
+    bootloader_file_in_boot_part = True
 
     # XXX: Here we define these things as dynamic properties because our
     # temporary hack to fix bug 697824 relies on changing the board's
@@ -903,8 +920,8 @@ class OmapConfig(BoardConfig):
                 cls.serial_tty = classproperty(lambda cls: 'ttyS2')
 
     @classmethod
-    def make_boot_files(cls, uboot_parts_dir, is_live, is_lowmem, consoles,
-                        chroot_dir, rootfs_uuid, boot_dir,
+    def make_boot_files(cls, bootloader_parts_dir, is_live, is_lowmem,
+                        consoles, chroot_dir, rootfs_id, boot_dir,
                         boot_device_or_file):
         # XXX: This is also part of our temporary hack to fix bug 697824; we
         # need to call set_appropriate_serial_tty() before doing anything that
@@ -912,8 +929,8 @@ class OmapConfig(BoardConfig):
         if cls.hwpack_format == HardwarepackHandler.FORMAT_1:
             cls.set_appropriate_serial_tty(chroot_dir)
         super(OmapConfig, cls).make_boot_files(
-            uboot_parts_dir, is_live, is_lowmem, consoles, chroot_dir,
-            rootfs_uuid, boot_dir, boot_device_or_file)
+            bootloader_parts_dir, is_live, is_lowmem, consoles, chroot_dir,
+            rootfs_id, boot_dir, boot_device_or_file)
 
     @classmethod
     def _make_boot_files(cls, boot_env, chroot_dir, boot_dir,
@@ -931,7 +948,7 @@ class OmapConfig(BoardConfig):
 
 
 class BeagleConfig(OmapConfig):
-    uboot_flavor = 'omap3_beagle'
+    bootloader_flavor = 'omap3_beagle'
     dtb_name = 'omap3-beagle.dtb'
     _serial_tty = 'ttyO2'
     _extra_serial_opts = 'console=tty0 console=%s,115200n8'
@@ -947,7 +964,7 @@ class BeagleConfig(OmapConfig):
 
 
 class OveroConfig(OmapConfig):
-    uboot_flavor = 'omap3_overo'
+    bootloader_flavor = 'omap3_overo'
     dtb_name = 'omap3-overo.dtb'
     _serial_tty = 'ttyO2'
     _extra_serial_opts = 'console=tty0 console=%s,115200n8'
@@ -962,7 +979,7 @@ class OveroConfig(OmapConfig):
 
 
 class PandaConfig(OmapConfig):
-    uboot_flavor = 'omap4_panda'
+    bootloader_flavor = 'omap4_panda'
     dtb_name = 'omap4-panda.dtb'
     _serial_tty = 'ttyO2'
     _extra_serial_opts = 'console=tty0 console=%s,115200n8'
@@ -978,8 +995,8 @@ class PandaConfig(OmapConfig):
 
 
 class IgepConfig(BeagleConfig):
-    uboot_in_boot_part = False
-    uboot_flavor = None
+    bootloader_file_in_boot_part = False
+    bootloader_flavor = None
     dtb_name = 'isee-igep-v2.dtb'
 
     @classmethod
@@ -1277,10 +1294,11 @@ class Mx5Config(BoardConfig):
         # XXX: delete this method when hwpacks V1 can die
         assert cls.hwpack_format == HardwarepackHandler.FORMAT_1
         with cls.hardwarepack_handler:
-            uboot_file = cls.get_file('u_boot', default=os.path.join(
-                    chroot_dir, 'usr', 'lib', 'u-boot', cls.uboot_flavor,
+            bootloader_file = cls.get_file('bootloader_file',
+                default=os.path.join(
+                    chroot_dir, 'usr', 'lib', 'u-boot', cls.bootloader_flavor,
                     'u-boot.imx'))
-            install_mx5_boot_loader(uboot_file, boot_device_or_file,
+            install_mx5_boot_loader(bootloader_file, boot_device_or_file,
                                     cls.LOADER_MIN_SIZE_S)
         make_uImage(cls.load_addr, k_img_data, boot_dir)
         make_uInitrd(i_img_data, boot_dir)
@@ -1306,28 +1324,28 @@ class Mx53Config(Mx5Config):
 
 
 class EfikamxConfig(Mx51Config):
-    uboot_flavor = 'efikamx'
+    bootloader_flavor = 'efikamx'
     dtb_name = 'genesi-efikamx.dtb'
 
 
 class EfikasbConfig(Mx51Config):
-    uboot_flavor = 'efikasb'
+    bootloader_flavor = 'efikasb'
     dtb_name = 'genesi-efikasb.dtb'
 
 
 class Mx51evkConfig(Mx51Config):
-    uboot_flavor = 'mx51evk'
+    bootloader_flavor = 'mx51evk'
     dtb_name = 'mx51-babbage.dtb'
 
 
 class Mx53LoCoConfig(Mx53Config):
-    uboot_flavor = 'mx53loco'
+    bootloader_flavor = 'mx53loco'
     dtb_name = 'mx53-loco.dtb'
 
 
 class VexpressConfig(BoardConfig):
-    uboot_flavor = 'ca9x4_ct_vxp'
-    uboot_in_boot_part = True
+    bootloader_flavor = 'ca9x4_ct_vxp'
+    bootloader_file_in_boot_part = True
     serial_tty = 'ttyAMA0'
     _extra_serial_opts = 'console=tty0 console=%s,38400n8'
     _live_serial_opts = 'serialtty=%s'
@@ -1424,8 +1442,7 @@ class SamsungConfig(BoardConfig):
         # XXX: delete this method when hwpacks V1 can die
         assert cls.hwpack_format == HardwarepackHandler.FORMAT_1
         cls.install_samsung_boot_loader(cls._get_samsung_spl(chroot_dir),
-                                        cls._get_samsung_uboot(chroot_dir),
-                                        boot_device_or_file)
+            cls._get_samsung_bootloader(chroot_dir), boot_device_or_file)
         env_size = cls.SAMSUNG_V310_ENV_LEN * SECTOR_SIZE
         env_file = make_flashable_env(boot_env, env_size)
         _dd(env_file, boot_device_or_file, seek=cls.SAMSUNG_V310_ENV_START)
@@ -1443,7 +1460,7 @@ class SamsungConfig(BoardConfig):
         # XXX: delete this method when hwpacks V1 can die
         assert cls.hwpack_format == HardwarepackHandler.FORMAT_1
         spl_dir = os.path.join(
-            chroot_dir, 'usr', 'lib', 'u-boot', cls.uboot_flavor)
+            chroot_dir, 'usr', 'lib', 'u-boot', cls.bootloader_flavor)
         old_spl_path = os.path.join(spl_dir, 'v310_mmc_spl.bin')
         new_spl_path = os.path.join(spl_dir, 'u-boot-mmc-spl.bin')
         new_new_spl_path = os.path.join(spl_dir, 'origen-spl.bin')
@@ -1464,13 +1481,13 @@ class SamsungConfig(BoardConfig):
         return spl_file
 
     @classmethod
-    def _get_samsung_uboot(cls, chroot_dir):
+    def _get_samsung_bootloader(cls, chroot_dir):
         # XXX: delete this method when hwpacks V1 can die
         assert cls.hwpack_format == HardwarepackHandler.FORMAT_1
-        uboot_file = os.path.join(
-            chroot_dir, 'usr', 'lib', 'u-boot', cls.uboot_flavor,
+        bootloader_file = os.path.join(
+            chroot_dir, 'usr', 'lib', 'u-boot', cls.bootloader_flavor,
             'u-boot.bin')
-        return uboot_file
+        return bootloader_file
 
     @classmethod
     def populate_raw_partition(cls, boot_device_or_file, chroot_dir):
@@ -1491,7 +1508,7 @@ class SamsungConfig(BoardConfig):
 
 
 class SMDKV310Config(SamsungConfig):
-    uboot_flavor = 'smdkv310'
+    bootloader_flavor = 'smdkv310'
     serial_tty = 'ttySAC1'
     _extra_serial_opts = 'console=%s,115200n8'
     kernel_addr = '0x40007000'
@@ -1503,10 +1520,10 @@ class SMDKV310Config(SamsungConfig):
     mmc_option = '0:2'
 
     @classmethod
-    def _get_boot_env(cls, is_live, is_lowmem, consoles, rootfs_uuid,
-                      d_img_data):
+    def _get_boot_env(cls, is_live, is_lowmem, consoles, rootfs_id,
+                      i_img_data, d_img_data):
         boot_env = super(SamsungConfig, cls)._get_boot_env(
-            is_live, is_lowmem, consoles, rootfs_uuid, d_img_data)
+            is_live, is_lowmem, consoles, rootfs_id, i_img_data, d_img_data)
 
         boot_env["ethact"] = "smc911x-0"
         boot_env["ethaddr"] = "00:40:5c:26:0a:5b"
@@ -1515,7 +1532,7 @@ class SMDKV310Config(SamsungConfig):
 
 
 class OrigenConfig(SamsungConfig):
-    uboot_flavor = 'origen'
+    bootloader_flavor = 'origen'
     serial_tty = 'ttySAC2'
     _extra_serial_opts = 'console=%s,115200n8'
     kernel_addr = '0x40007000'
@@ -1778,7 +1795,7 @@ def install_omap_boot_loader(chroot_dir, boot_disk, cls):
             default = _get_mlo_file(chroot_dir)
         except AssertionError:
             default = None
-        mlo_file = cls.get_file('spl', default=default)
+        mlo_file = cls.get_file('spl_file', default=default)
         cmd_runner.run(["cp", "-v", mlo_file, boot_disk], as_root=True).wait()
         # XXX: Is this really needed?
         cmd_runner.run(["sync"]).wait()
